/*
  Universel NVM (Nederlandse Vereniging voor Modelbouw) 14 Functies MultiSwitch Encoder Module V1.0
  NVM - Maarten Flederus / Written by Marten de Groot "april 2016"
  
Pin Connections input side:                  (This will happen, after a Switch is released:)
  S1_Down is connected to Digital pin 0 = D0 (by a long press, 2 Pulses)
  S1_Up   is connected to Digital pin 1 = D1 (by a long press, 3 Pulses)

  S2_Down is connected to Digital pin 2 = D2 (by a long press, 4 Pulses or by a short press, 10 Pulses)
  S2_Up   is connected to Digital pin 3 = D3 (by a long press, 5 Pulses or by a short press, 11 Pulses)

  S3_Down is connected to Digital pin 4 = D4 (by a long press, 6 Pulses or by a short press, 12 Pulses)
  S3_Up   is connected to Digital pin 5 = D5 (by a long press, 7 Pulses or by a short press, 13 Pulses)

  S4_Down is connected to Digital pin 6 = D6 (by a long press, 8 Pulses or by a short press, 14 Pulses)
  S4_Up   is connected to Digital pin 7 = D7 (by a long press, 9 Pulses or by a short press, 15 Pulses)

  All Switches will read as a 8-bit Binary format, by all Switches Off it looks like this: B00000000

  The EnableReverse pin is used to invert the Puls state between High to low or from Low to High

Pin Connection(s) output side:
  PulsOutPin is connected to Digital pin 12
  Led is connected to Digital pin 13 (default led pin)
*/

#define PulsTime         40  // This time is needed to make sure we will read at least one time the Up or down puls (1.0uS or 2.0uS at receiver side)
#define ResetPulsTime   160  // This Reset time is needed, to make sure the MultiSwitch Decoder reads this as a Reset and starts over again.
#define LongSwitchTime   80  // Long press function 40 x 10ms = 0.4 seconds
#define MomentSwitchTime 10  // Long press function 10 x 10ms = 0.1 seconds

#define Jumper1pin        8 //
#define PulsOutPin       12  // This pin is connected to a Opto-Coupler and switch a two line switch input from a basic tranceiver.
#define Led              13  // The Led showes the state during the long press (led go's off after 1sec and the led go's off during the pulses)
 
int PulsCount           = 0; // Depents to which Switch is active 
int ShortPress          = 0;
byte SwitchesState;
int Jumper1;

// the setup routine runs once when you press reset or during a normal start up:
void setup() {
  // set all PORTD pins and EnableReverse to a input, including the inside pullup resistor:
  pinMode(0, INPUT_PULLUP);
  pinMode(1, INPUT_PULLUP);
  pinMode(2, INPUT_PULLUP);
  pinMode(3, INPUT_PULLUP);
  pinMode(4, INPUT_PULLUP);
  pinMode(5, INPUT_PULLUP);
  pinMode(6, INPUT_PULLUP);
  pinMode(7, INPUT_PULLUP);
  
  pinMode(Jumper1pin, INPUT_PULLUP); //Pin 8 as input

  Jumper1 = digitalRead(Jumper1pin);  // Check if there is a Jumper placed
  
  // Set the PulsOutPin and the Led as a output pin:
  pinMode(PulsOutPin, OUTPUT);
  pinMode(Led, OUTPUT);

  digitalWrite(Led, HIGH); // Turn on the Led
  digitalWrite(PulsOutPin, LOW);
}

void PinRead() // Reading all the pins in one time (go's faster and we can use the Binary value for more options later
{
  SwitchesState = PIND;
}

// the loop routine runs over and over again forever:
void loop() {
  // read all the input pins:
  PinRead(); // After reading all the pins, if one Switch is active.....:
  
  Jumper1 = digitalRead(Jumper1pin);  // Check if there is a Jumper placed
  
  switch (SwitchesState){
  case B11111111:
    if (PulsCount > 0) // After releasing a Switch, if the PulsCount is higher as 0, if yes it starts pulsing.
    {
      digitalWrite(Led, LOW);
      for (int i=1; i <= PulsCount; i++)
        {
           digitalWrite(PulsOutPin, HIGH);
           delay(PulsTime);
           digitalWrite(PulsOutPin, LOW);
           delay(PulsTime);
        }
      PulsCount = 0;   // Reset the PulsCount to 0
      ShortPress = 0;  // Reset the ShortPress to 0
      digitalWrite(Led, HIGH);  // Set the Led on
      
      delay(ResetPulsTime); // Waith for xxx ms to give the decoder the time to stop the PulsCounting.
    }
    else if (PulsCount == 0)
      {
       digitalWrite(PulsOutPin, LOW);
      }
    break;
  case B11111101: // Switch S1_Down
    for (int i=0; i <= LongSwitchTime; i++){
      delay(10);
      PinRead();
      if (SwitchesState == B11111111){
        ShortPress = 1;
      break;}
    }
    switch(ShortPress)
    {
      case 0: // LongPress
        if(Jumper1 == HIGH){
        PulsCount = 9;} // NVM count
        else{
        PulsCount = 9;  // USM count
        }
      break;
      case 1: // ShortPress
        if(Jumper1 == HIGH){
        PulsCount = 1;} // NVM count
        else{
        PulsCount = 1;  // USM count
        }
      break;
    }
    break;
  case B11111110: // Switch S1_Up
    for (int i=0; i <= LongSwitchTime; i++){
      delay(10);
      PinRead();
      if (SwitchesState == B11111111){
        ShortPress = 1;
      break;}
    }
    switch(ShortPress)
    {
      case 0:
        if(Jumper1 == HIGH){
        PulsCount = 10;}
        else{
        PulsCount = 10;
        }
      break;
      case 1:
        if(Jumper1 == HIGH){
        PulsCount = 2;}
        else{
        PulsCount = 2;
        }
      break;
    }
    break;
  case B11111011: // Switch S2_Down
    for (int i=0; i <= LongSwitchTime; i++){
      delay(10);
      PinRead();
      if (SwitchesState == B11111111){
        ShortPress = 1;
      break;}
    }
    switch(ShortPress)
    {
      case 0:
        if(Jumper1 == HIGH){
        PulsCount = 11;}
        else{
        PulsCount = 11;
        }
      break;
      case 1:
        if(Jumper1 == HIGH){
        PulsCount = 3;}
        else{
        PulsCount = 3;
        }
      break;
    }
    break;
  case B11110111: // Switch S2_Up
    for (int i=0; i <= LongSwitchTime; i++){
      delay(10);
      PinRead();
      if (SwitchesState == B11111111){
        ShortPress = 1;
      break;}
    }
    switch(ShortPress)
    {
      case 0:
        if(Jumper1 == HIGH){
        PulsCount = 12;}
        else{
        PulsCount = 12;
        }
      break;
      case 1:
        if(Jumper1 == HIGH){
        PulsCount = 4;}
        else{
        PulsCount = 4;
        }
      break;
    }
    break;
  case B11101111: // Switch S3_Down
    for (int i=0; i <= LongSwitchTime; i++){
      delay(10);
      PinRead();
      if (SwitchesState == B11111111){
        ShortPress = 1;
      break;}
    }
    switch(ShortPress)
    {
      case 0:
        if(Jumper1 == HIGH){
        PulsCount = 13;}
        else{
        PulsCount = 13;
        }
      break;
      case 1:
        if(Jumper1 == HIGH){
        PulsCount = 5;}
        else{
        PulsCount = 5;
        }
      break;
    }
    break;
  case B11011111: // Switch S3_Up
    for (int i=0; i <= LongSwitchTime; i++){
      delay(10);
      PinRead();
      if (SwitchesState == B11111111){
        ShortPress = 1;
      break;}
    }
    switch(ShortPress)
    {
      case 0:
        if(Jumper1 == HIGH){
        PulsCount = 14;}
        else{
        PulsCount = 14;
        }
      break;
      case 1:
        if(Jumper1 == HIGH){
        PulsCount = 6;}
        else{
        PulsCount = 6;
        }
      break;
    }
    break;
  case B10111111: // Switch S4_Down
    for (int i=0; i <= LongSwitchTime; i++){
      delay(10);
      PinRead();
      if (SwitchesState == B11111111){
        ShortPress = 1;
      break;}
    }
    switch(ShortPress)
    {
      case 0:
        if(Jumper1 == HIGH){
        PulsCount = 15;}
        else{
        PulsCount = 15;
        }
      break;
      case 1:
        if(Jumper1 == HIGH){
        PulsCount = 7;}
        else{
        PulsCount = 7;
        }
      break;
    }
    break;
  case B01111111: // Switch S4_Up
    for (int i=0; i <= LongSwitchTime; i++){
      delay(10);
      PinRead();
      if (SwitchesState == B11111111){
        ShortPress = 1;
      break;}
      }
    switch(ShortPress)
    {
      case 0:
        if(Jumper1 == HIGH){
        PulsCount = 16;}
        else{
        PulsCount = 16;
        }
      break;
      case 1:
        if(Jumper1 == HIGH){
        PulsCount = 8;}
        else{
        PulsCount = 8;
        }
      break;
    }
    break;
  default:
  // there is more as one switch active, in this "Case" do nothing
  // Er zijn nu meerdere schakelaars aan, in dit geval voer niks uit.
    break;
  }
  while(SwitchesState !=  B11111111){ // This while is needed, to waith until all switches are released again.
    digitalWrite(Led, LOW); // This Led indigates the LongPress actions.
    PinRead();
    delay(20);  // Small delay to check the switches each time  
  }
 }



